@@ -91,7 +91,7 @@ static const char *defaultDataDir_App_ = "~/.config/lagrange";
 #endif
 #if defined (iPlatformHaiku)
 #define EMB_BIN "./resources.lgr"
-static const char *defaultDataDir_App_ = "~/config/settings";
+static const char *defaultDataDir_App_ = "~/config/settings/lagrange";
 #endif
 #if defined (LAGRANGE_EMB_BIN) /* specified in build config */
 #  undef EMB_BIN
@@ -129,7 +129,7 @@ struct Impl_App {
     iTime        lastDropTime; /* for detecting drops of multiple items */
     int          autoReloadTimer;
     iPeriodic    periodic;
-    int          warmupFrames; /* forced refresh just after resuming from background */
+    int          warmupFrames; /* forced refresh just after resuming from background; FIXME: shouldn't be needed */
     /* Preferences: */
     iBool        commandEcho;         /* --echo */
     iBool        forceSoftwareRender; /* --sw */
@@ -180,8 +180,8 @@ static iString *serializePrefs_App_(const iApp *d) {
         w = d->window->place.normalRect.size.x;
         h = d->window->place.normalRect.size.y;
         appendFormat_String(str, "window.setrect width:%d height:%d coord:%d %d\n", w, h, x, y);
-        appendFormat_String(str, "sidebar.width arg:%d\n", width_SidebarWidget(sidebar));
-        appendFormat_String(str, "sidebar2.width arg:%d\n", width_SidebarWidget(sidebar2));
+        appendFormat_String(str, "sidebar.width arg:%f gaps:1\n", width_SidebarWidget(sidebar));
+        appendFormat_String(str, "sidebar2.width arg:%f gaps:1\n", width_SidebarWidget(sidebar2));
         /* On macOS, maximization should be applied at creation time or the window will take
            a moment to animate to its maximized size. */
 #if defined (LAGRANGE_CUSTOM_FRAME)
@@ -226,6 +226,8 @@ static iString *serializePrefs_App_(const iApp *d) {
     /* TODO: Set up an array of booleans in Prefs and do these in a loop. */
     appendFormat_String(str, "prefs.mono.gemini.changed arg:%d\n", d->prefs.monospaceGemini);
     appendFormat_String(str, "prefs.mono.gopher.changed arg:%d\n", d->prefs.monospaceGopher);
+    appendFormat_String(str, "prefs.boldlink.dark.changed arg:%d\n", d->prefs.boldLinkDark);
+    appendFormat_String(str, "prefs.boldlink.light.changed arg:%d\n", d->prefs.boldLinkLight);
     appendFormat_String(str, "prefs.biglede.changed arg:%d\n", d->prefs.bigFirstParagraph);
     appendFormat_String(str, "prefs.plaintext.wrap.changed arg:%d\n", d->prefs.plainTextWrap);
     appendFormat_String(str, "prefs.sideicon.changed arg:%d\n", d->prefs.sideIcon);
@@ -300,6 +302,7 @@ static const iString *prefsFileName_(void) {
 
 static void loadPrefs_App_(iApp *d) {
     iUnused(d);
+    iBool haveCA = iFalse;
     /* Create the data dir if it doesn't exist yet. */
     makeDirs_Path(collectNewCStr_String(dataDir_App_()));
     iFile *f = new_File(prefsFileName_());
@@ -324,6 +327,7 @@ static void loadPrefs_App_(iApp *d) {
                 /* Background requests may be started before these commands would get
                    handled via the event loop. */
                 handleCommand_App(cmd);
+                haveCA = iTrue;
             }
             else if (equal_Command(cmd, "customframe")) {
                 d->prefs.customFrame = arg_Command(cmd);
@@ -345,8 +349,8 @@ static void loadPrefs_App_(iApp *d) {
         }
         delete_String(str);
     }
-    else {
-        /* default preference values */
+    if (!haveCA) {
+        /* Default CA setup. */
         setCACertificates_TlsRequest(&d->prefs.caFile, &d->prefs.caPath);
     }
 #if !defined (LAGRANGE_CUSTOM_FRAME)
@@ -945,6 +949,7 @@ void processEvents_App(enum iAppEventMode eventMode) {
                     }
                     d->lastDropTime = now_Time();
                     if (startsWithCase_CStr(ev.drop.file, "gemini:") ||
+                        startsWithCase_CStr(ev.drop.file, "gopher:") ||
                         startsWithCase_CStr(ev.drop.file, "file:")) {
                         postCommandf_App("~open newtab:%d url:%s", newTab, ev.drop.file);
                     }
@@ -1017,6 +1022,16 @@ void processEvents_App(enum iAppEventMode eventMode) {
                     /* There may be a key bindings for this. */
                     wasUsed = processEvent_Keys(&ev);
                 }
+                if (!wasUsed) {
+                    /* Focus cycling. */
+                    if (ev.type == SDL_KEYDOWN && ev.key.keysym.sym == SDLK_TAB) {
+                        setFocus_Widget(findFocusable_Widget(focus_Widget(),
+                                                             ev.key.keysym.mod & KMOD_SHIFT
+                                                                 ? backward_WidgetFocusDir
+                                                                 : forward_WidgetFocusDir));
+                        wasUsed = iTrue;
+                    }
+                }
                 if (ev.type == SDL_USEREVENT && ev.user.code == command_UserEventCode) {
 #if defined (iPlatformAppleDesktop)
                     handleCommand_MacOS(command_UserEvent(&ev));
@@ -1697,6 +1712,19 @@ iBool handleCommand_App(const char *cmd) {
         postCommand_App("window.unfreeze");
         return iTrue;
     }
+    else if (equal_Command(cmd, "prefs.boldlink.dark.changed") ||
+             equal_Command(cmd, "prefs.boldlink.light.changed")) {
+        const iBool isSet = (arg_Command(cmd) != 0);
+        if (startsWith_CStr(cmd, "prefs.boldlink.dark")) {
+            d->prefs.boldLinkDark = isSet;
+        }
+        else {
+            d->prefs.boldLinkLight = isSet;
+        }
+        resetFonts_Text(); /* clear the glyph cache */
+        postCommand_App("font.changed");
+        return iTrue;
+    }
     else if (equal_Command(cmd, "prefs.biglede.changed")) {
         d->prefs.bigFirstParagraph = arg_Command(cmd) != 0;
         postCommand_App("document.layout.changed");
@@ -1938,6 +1966,12 @@ iBool handleCommand_App(const char *cmd) {
         setFlags_Widget(findChild_Widget(dlg, "prefs.mono.gopher"),
                         selected_WidgetFlag,
                         d->prefs.monospaceGopher);
+        setFlags_Widget(findChild_Widget(dlg, "prefs.boldlink.dark"),
+                        selected_WidgetFlag,
+                        d->prefs.boldLinkDark);
+        setFlags_Widget(findChild_Widget(dlg, "prefs.boldlink.light"),
+                        selected_WidgetFlag,
+                        d->prefs.boldLinkLight);
         setFlags_Widget(
             findChild_Widget(dlg, format_CStr("prefs.linewidth.%d", d->prefs.lineWidth)),
             selected_WidgetFlag,
