@@ -161,14 +161,14 @@ static iBool handleRootCommands_(iWidget *root, const char *cmd) {
                         isPortrait_App());
         if (isLandscape_App()) {
             addChildPos_Widget(findChild_Widget(root, "tabs.content"), iClob(sidebar), front_WidgetAddPos);
-            setWidth_SidebarWidget(sidebar, 73 * gap_UI);
+            setWidth_SidebarWidget(sidebar, 73.0f);
             if (isVisible_Widget(findWidget_App("sidebar2"))) {
                 postCommand_App("sidebar2.toggle");
             }
         }
         else {
             addChildPos_Widget(findChild_Widget(root, "stack"), iClob(sidebar), back_WidgetAddPos);
-            setWidth_SidebarWidget(sidebar, width_Widget(root));
+            setWidth_SidebarWidget(sidebar, (float) width_Widget(root) / (float) gap_UI);
         }
         return iFalse;
     }
@@ -1276,19 +1276,36 @@ void drawWhileResizing_Window(iWindow *d, int w, int h) {
 }
 
 static float pixelRatio_Window_(const iWindow *d) {
-#if defined (iPlatformMsys)
-    iUnused(d);
-    return desktopDPI_Win32();
-#elif defined (iPlatformLinux)
-    float vdpi = 0.0f;
-    SDL_GetDisplayDPI(SDL_GetWindowDisplayIndex(d->win), NULL, NULL, &vdpi);
-    const float factor = vdpi / 96.0f;
-    return iMax(1.0f, factor);
-#else
     int dx, x;
     SDL_GetRendererOutputSize(d->render, &dx, NULL);
     SDL_GetWindowSize(d->win, &x, NULL);
     return (float) dx / (float) x;
+}
+
+#if defined (iPlatformApple)
+#   define baseDPI_Window   113.5f
+#else
+#   define baseDPI_Window   96.0f
+#endif
+
+static float displayScale_Window_(const iWindow *d) {
+#if defined (iPlatformApple)
+    iUnused(d);
+    /* Apple UI sizes are fixed and only scaled by pixel ratio. */
+    /* TODO: iOS text size setting? */
+    return 1.0f;
+#elif defined (iPlatformMsys)
+    iUnused(d);
+    return desktopDPI_Win32();
+#else
+    float vdpi = 0.0f;
+    SDL_GetDisplayDPI(SDL_GetWindowDisplayIndex(d->win), NULL, NULL, &vdpi);
+//    printf("DPI: %f\n", vdpi);
+    const float factor = vdpi / baseDPI_Window / pixelRatio_Window_(d);
+    if (factor < 0.5f) {
+        return 1.0f; /* seems invalid */
+    }
+    return iMax(0.75f, factor);
 #endif
 }
 
@@ -1443,14 +1460,12 @@ void init_Window(iWindow *d, iRect rect) {
 #endif
     }
     drawBlank_Window_(d);
-    d->uiScale = initialUiScale_;
-    d->pixelRatio = pixelRatio_Window_(d);
-    setPixelRatio_Metrics(d->pixelRatio * d->uiScale);
+    d->pixelRatio   = pixelRatio_Window_(d); /* point/pixel conversion */
+    d->displayScale = displayScale_Window_(d);
+    d->uiScale      = initialUiScale_;
+    setScale_Metrics(d->pixelRatio * d->displayScale * d->uiScale);
 #if defined (iPlatformMsys)
-    SDL_Rect usable;
-    SDL_GetDisplayUsableBounds(0, &usable);
-    SDL_SetWindowMaximumSize(d->win, usable.w, usable.h);
-    SDL_SetWindowMinimumSize(d->win, minSize.x * d->pixelRatio, minSize.y * d->pixelRatio);
+    SDL_SetWindowMinimumSize(d->win, minSize.x * d->displayScale, minSize.y * d->displayScale);
     useExecutableIconResource_SDLWindow(d->win);
 #endif
 #if defined (iPlatformLinux)
@@ -1473,7 +1488,7 @@ void init_Window(iWindow *d, iRect rect) {
     setId_Widget(d->root, "root");
     init_Text(d->render);
     setupUserInterface_Window(d);
-    postCommand_App("bindings.changed"); /* update from bindings */
+    postCommand_App("~bindings.changed"); /* update from bindings */
     updateRootSize_Window_(d, iFalse);
     /* Load the border shadow texture. */ {
         SDL_Surface *surf = loadImage_(&imageShadow_Embedded, 0);
@@ -1581,15 +1596,24 @@ static iBool unsnap_Window_(iWindow *d, const iInt2 *newPos) {
 
 static void notifyMetricsChange_Window_(const iWindow *d) {
     /* Dynamic UI metrics change. Widgets need to update themselves. */
-    setPixelRatio_Metrics(d->pixelRatio * d->uiScale);
+    setScale_Metrics(d->pixelRatio * d->displayScale * d->uiScale);
     resetFonts_Text();
     postCommand_App("metrics.changed");
 }
 
 static void checkPixelRatioChange_Window_(iWindow *d) {
+    iBool wasChanged = iFalse;
     const float ratio = pixelRatio_Window_(d);
     if (iAbs(ratio - d->pixelRatio) > 0.001f) {
         d->pixelRatio = ratio;
+        wasChanged = iTrue;
+    }
+    const float scale = displayScale_Window_(d);
+    if (iAbs(scale - d->displayScale) > 0.001f) {
+        d->displayScale = scale;
+        wasChanged = iTrue;
+    }
+    if (wasChanged) {
         notifyMetricsChange_Window_(d);
     }
 }
@@ -1769,12 +1793,8 @@ iBool processEvent_Window(iWindow *d, const SDL_Event *ev) {
             SDL_Event event = *ev;
             if (event.type == SDL_USEREVENT && isCommand_UserEvent(ev, "window.unfreeze")) {
                 d->isDrawFrozen = iFalse;
-                /* When the window is shown for the first time, ensure glyphs get
-                   re-cached correctly. */
                 if (SDL_GetWindowFlags(d->win) & SDL_WINDOW_HIDDEN) {
                     SDL_ShowWindow(d->win);
-                    resetFonts_Text();
-                    postCommand_App("theme.changed");
                 }
                 postRefresh_App();
                 return iTrue;
@@ -1995,13 +2015,7 @@ iInt2 visibleRootSize_Window(const iWindow *d) {
 }
 
 iInt2 coord_Window(const iWindow *d, int x, int y) {
-#if defined (iPlatformMsys) || defined (iPlatformLinux)
-    /* On Windows, surface coordinates are in pixels. */
-    return init_I2(x, y);
-#else
-    /* Coordinates are in points. */
     return mulf_I2(init_I2(x, y), d->pixelRatio);
-#endif
 }
 
 iInt2 mouseCoord_Window(const iWindow *d) {
