@@ -67,7 +67,8 @@ struct Impl_Window {
     uint32_t      focusGainedAt;
     SDL_Renderer *render;
     iWidget *     root;
-    float         pixelRatio;
+    float         pixelRatio; /* conversion between points and pixels, e.g., coords, window size */
+    float         displayScale; /* DPI-based scaling factor of current display, affects uiScale only */
     float         uiScale;
     uint32_t      frameTime;
     double        presentTime;
@@ -113,3 +114,8 @@ iWindow *   get_Window              (void);
 #if defined (LAGRANGE_CUSTOM_FRAME)
 SDL_HitTestResult hitTest_Window(const iWindow *d, iInt2 pos);
 #endif
+
+iLocalDef iBool isExposed_Window(const iWindow *d) {
+    iAssert(d);
+    return d->isExposed;
+}
